@@ -28,13 +28,14 @@ type MatchTest struct {
 var onWindows = runtime.GOOS == "windows"
 
 var matchTests = []MatchTest{
+	{"", "", true, false, nil, true, false, true, true, 0, 0},
 	{"*", "", true, true, nil, false, false, true, false, 0, 0},
 	{"*", "/", false, false, nil, false, false, true, false, 0, 0},
 	{"/*", "/", true, true, nil, false, false, true, false, 0, 0},
 	{"/*", "/debug/", false, false, nil, false, false, true, false, 0, 0},
 	{"/*", "//", false, false, nil, false, false, true, false, 0, 0},
 	{"abc", "abc", true, true, nil, false, false, true, true, 1, 1},
-	{"*", "abc", true, true, nil, false, false, true, true, 22, 17},
+	{"*", "abc", true, true, nil, false, false, true, true, 23, 18},
 	{"*c", "abc", true, true, nil, false, false, true, true, 2, 2},
 	{"*/", "a/", true, true, nil, false, false, true, false, 0, 0},
 	{"a*", "a", true, true, nil, false, false, true, true, 9, 9},
@@ -97,9 +98,13 @@ var matchTests = []MatchTest{
 	{"ad[", "ab", false, false, ErrBadPattern, false, false, true, true, 0, 0},
 	{"*x", "xxx", true, true, nil, false, false, true, true, 4, 4},
 	{"[abc]", "b", true, true, nil, false, false, true, true, 3, 3},
+	{"[abc123]", "1", true, true, nil, false, false, true, true, 4, 4},
+	{"[a-z0-9]", "1", true, true, nil, false, false, true, true, 7, 7},
 	{"**", "", true, true, nil, false, false, false, false, 38, 38},
 	{"a/**", "a", true, true, nil, false, false, false, true, 7, 7},
+	{"a/**/", "a", true, true, nil, false, false, false, true, 4, 4},
 	{"a/**", "a/", true, true, nil, false, false, false, false, 7, 7},
+	{"a/**/", "a/", true, true, nil, false, false, false, false, 4, 4},
 	{"a/**", "a/b", true, true, nil, false, false, false, true, 7, 7},
 	{"a/**", "a/b/c", true, true, nil, false, false, false, true, 7, 7},
 	{"**/c", "c", true, true, nil, !onWindows, false, false, true, 5, 4},
@@ -139,6 +144,7 @@ var matchTests = []MatchTest{
 	{"**/*.txt", "abc/【test】.txt", true, true, nil, !onWindows, false, false, true, 1, 1},
 	{"**/【*", "abc/【test】.txt", true, true, nil, !onWindows, false, false, true, 1, 1},
 	{"**/{a,b}", "a/b", true, true, nil, !onWindows, false, false, true, 5, 5},
+	{"a/*/*/d", "a/b/c/d", true, true, nil, false, false, true, true, 1, 1},
 	// unfortunately, io/fs can't handle this, so neither can Glob =(
 	{"broken-symlink", "broken-symlink", true, true, nil, false, false, true, false, 1, 1},
 	{"broken-symlink/*", "a", false, false, nil, false, true, true, true, 0, 0},
@@ -146,16 +152,19 @@ var matchTests = []MatchTest{
 	{"working-symlink/c/*", "working-symlink/c/d", true, true, nil, false, false, true, !onWindows, 1, 1},
 	{"working-sym*/*", "working-symlink/c", true, true, nil, false, false, true, !onWindows, 1, 1},
 	{"b/**/f", "b/symlink-dir/f", true, true, nil, false, false, false, !onWindows, 2, 2},
-	{"e/**", "e/**", true, true, nil, false, false, false, !onWindows, 11, 6},
-	{"e/**", "e/*", true, true, nil, false, false, false, !onWindows, 11, 6},
-	{"e/**", "e/?", true, true, nil, false, false, false, !onWindows, 11, 6},
-	{"e/**", "e/[", true, true, nil, false, false, false, true, 11, 6},
-	{"e/**", "e/]", true, true, nil, false, false, false, true, 11, 6},
-	{"e/**", "e/[]", true, true, nil, false, false, false, true, 11, 6},
-	{"e/**", "e/{", true, true, nil, false, false, false, true, 11, 6},
-	{"e/**", "e/}", true, true, nil, false, false, false, true, 11, 6},
-	{"e/**", "e/\\", true, true, nil, false, false, false, !onWindows, 11, 6},
-	{"e/*", "e/*", true, true, nil, false, false, true, !onWindows, 10, 5},
+	{"*/symlink-dir/*", "b/symlink-dir/f", true, true, nil, !onWindows, false, true, !onWindows, 2, 2},
+	{"e/\\[x\\]/*", "e/[x]/[y]", true, true, nil, false, false, true, !onWindows, 1, 1},
+	{"e/\\[x\\]/*/z", "e/[x]/[y]/z", true, true, nil, false, false, true, !onWindows, 1, 1},
+	{"e/**", "e/**", true, true, nil, false, false, false, !onWindows, 14, 9},
+	{"e/**", "e/*", true, true, nil, false, false, false, !onWindows, 14, 9},
+	{"e/**", "e/?", true, true, nil, false, false, false, !onWindows, 14, 9},
+	{"e/**", "e/[", true, true, nil, false, false, false, true, 14, 9},
+	{"e/**", "e/]", true, true, nil, false, false, false, true, 14, 9},
+	{"e/**", "e/[]", true, true, nil, false, false, false, true, 14, 9},
+	{"e/**", "e/{", true, true, nil, false, false, false, true, 14, 9},
+	{"e/**", "e/}", true, true, nil, false, false, false, true, 14, 9},
+	{"e/**", "e/\\", true, true, nil, false, false, false, !onWindows, 14, 6},
+	{"e/*", "e/*", true, true, nil, false, false, true, !onWindows, 11, 5},
 	{"e/?", "e/?", true, true, nil, false, false, true, !onWindows, 7, 4},
 	{"e/?", "e/*", true, true, nil, false, false, true, !onWindows, 7, 4},
 	{"e/?", "e/[", true, true, nil, false, false, true, true, 7, 4},
@@ -188,6 +197,18 @@ var matchTests = []MatchTest{
 	{"nopermission/file", "nopermission/file", true, false, nil, true, false, true, !onWindows, 0, 0},
 }
 
+// Calculate the number of results that we expect
+// WithFilesOnly at runtime and memoize them here
+var numResultsFilesOnly []int
+
+// Calculate the number of results that we expect
+// WithNoFollow at runtime and memoize them here
+var numResultsNoFollow []int
+
+// Calculate the number of results that we expect with all
+// of the options enabled at runtime and memoize them here
+var numResultsAllOpts []int
+
 func TestValidatePattern(t *testing.T) {
 	for idx, tt := range matchTests {
 		testValidatePatternWith(t, idx, tt)
@@ -258,6 +279,56 @@ func BenchmarkGoMatch(b *testing.B) {
 	}
 }
 
+func TestMatchUnvalidated(t *testing.T) {
+	for idx, tt := range matchTests {
+		testMatchUnvalidatedWith(t, idx, tt)
+	}
+
+	// Not only test that they get the right matches, but make sure Unvalidated is properly *skipping* validation
+	unvalidatedTests := []struct {
+		pattern, testPath      string // a pattern and path to test the pattern on
+		expectedErrValidated   error  // the error expected if the pattern was being validated
+		expectedErrUnvalidated error  // the error expected if the pattern was not being validated
+	}{
+		{"", "", nil, nil},
+		{"*", "", nil, nil},
+		{"a[", "a", ErrBadPattern, nil},          // End early because got to end of match; do not need to validate the rest
+		{"a[", "b", ErrBadPattern, nil},          // End early because failed to match; do not need to validate the rest
+		{"[", "a", ErrBadPattern, ErrBadPattern}, // Error right up front, needs to fail whether validate or not
+	}
+	for idx, tt := range unvalidatedTests {
+		_, errValidated := matchWithSeparator(tt.pattern, tt.testPath, '/', true)
+		_, errUnvalidated := matchWithSeparator(tt.pattern, tt.testPath, '/', false)
+		if errValidated != tt.expectedErrValidated {
+			t.Errorf("#%v. Validated error of Match(%#q, %#q) = %v want %v", idx, tt.pattern, tt.testPath, errValidated, tt.expectedErrValidated)
+		}
+		if errUnvalidated != tt.expectedErrUnvalidated {
+			t.Errorf("#%v. Unvalidated error of Match(%#q, %#q) = %v want %v", idx, tt.pattern, tt.testPath, errUnvalidated, tt.expectedErrUnvalidated)
+		}
+	}
+}
+
+func testMatchUnvalidatedWith(t *testing.T, idx int, tt MatchTest) {
+	defer func() {
+		if r := recover(); r != nil {
+			t.Errorf("#%v. MatchUnvalidated(%#q, %#q) panicked: %#v", idx, tt.pattern, tt.testPath, r)
+		}
+	}()
+
+	// MatchUnvalidated() always uses "/" as the separator
+	ok := MatchUnvalidated(tt.pattern, tt.testPath)
+	if ok != tt.shouldMatch {
+		t.Errorf("#%v. MatchUnvalidated(%#q, %#q) = %v want %v", idx, tt.pattern, tt.testPath, ok, tt.shouldMatch)
+	}
+
+	if tt.isStandard {
+		stdOk, _ := path.Match(tt.pattern, tt.testPath)
+		if ok != stdOk {
+			t.Errorf("#%v. MatchUnvalidated(%#q, %#q) != path.Match(...). Got %v want %v", idx, tt.pattern, tt.testPath, ok, stdOk)
+		}
+	}
+}
+
 func TestPathMatch(t *testing.T) {
 	for idx, tt := range matchTests {
 		// Even though we aren't actually matching paths on disk, we are using
@@ -366,8 +437,16 @@ func TestGlobWithFailOnPatternNotExist(t *testing.T) {
 	doGlobTest(t, WithFailOnPatternNotExist())
 }
 
+func TestGlobWithFilesOnly(t *testing.T) {
+	doGlobTest(t, WithFilesOnly())
+}
+
+func TestGlobWithNoFollow(t *testing.T) {
+	doGlobTest(t, WithNoFollow())
+}
+
 func TestGlobWithAllOptions(t *testing.T) {
-	doGlobTest(t, WithFailOnIOErrors(), WithFailOnPatternNotExist())
+	doGlobTest(t, WithFailOnIOErrors(), WithFailOnPatternNotExist(), WithFilesOnly(), WithNoFollow())
 }
 
 func doGlobTest(t *testing.T, opts ...GlobOption) {
@@ -406,8 +485,16 @@ func TestGlobWalkWithFailOnPatternNotExist(t *testing.T) {
 	doGlobWalkTest(t, WithFailOnPatternNotExist())
 }
 
+func TestGlobWalkWithFilesOnly(t *testing.T) {
+	doGlobWalkTest(t, WithFilesOnly())
+}
+
+func TestGlobWalkWithNoFollow(t *testing.T) {
+	doGlobWalkTest(t, WithNoFollow())
+}
+
 func TestGlobWalkWithAllOptions(t *testing.T) {
-	doGlobWalkTest(t, WithFailOnIOErrors(), WithFailOnPatternNotExist())
+	doGlobWalkTest(t, WithFailOnIOErrors(), WithFailOnPatternNotExist(), WithFilesOnly(), WithNoFollow())
 }
 
 func doGlobWalkTest(t *testing.T, opts ...GlobOption) {
@@ -459,6 +546,14 @@ func TestFilepathGlobWithFailOnPatternNotExist(t *testing.T) {
 	doFilepathGlobTest(t, WithFailOnPatternNotExist())
 }
 
+func TestFilepathGlobWithFilesOnly(t *testing.T) {
+	doFilepathGlobTest(t, WithFilesOnly())
+}
+
+func TestFilepathGlobWithNoFollow(t *testing.T) {
+	doFilepathGlobTest(t, WithNoFollow())
+}
+
 func doFilepathGlobTest(t *testing.T, opts ...GlobOption) {
 	glob := newGlob(opts...)
 	fsys := os.DirFS("test")
@@ -470,7 +565,10 @@ func doFilepathGlobTest(t *testing.T, opts ...GlobOption) {
 	os.Chdir("test")
 
 	for idx, tt := range matchTests {
-		if tt.testOnDisk {
+		// Patterns ending with a slash are treated semantically different by
+		// FilepathGlob vs Glob because FilepathGlob runs filepath.Clean, which
+		// will remove the trailing slash.
+		if tt.testOnDisk && !strings.HasSuffix(tt.pattern, "/") {
 			ttmod := tt
 			ttmod.pattern = filepath.FromSlash(tt.pattern)
 			ttmod.testPath = filepath.FromSlash(tt.testPath)
@@ -520,11 +618,20 @@ func verifyGlobResults(t *testing.T, idx int, fn string, tt MatchTest, g *glob,
 		if onWindows {
 			numResults = tt.winNumResults
 		}
+		if g.filesOnly {
+			if g.noFollow {
+				numResults = numResultsAllOpts[idx]
+			} else {
+				numResults = numResultsFilesOnly[idx]
+			}
+		} else if g.noFollow {
+			numResults = numResultsNoFollow[idx]
+		}
 
 		if len(matches) != numResults {
 			t.Errorf("#%v. %v(%#q, %#v) = %#v - should have %#v results, got %#v", idx, fn, tt.pattern, g, matches, numResults, len(matches))
 		}
-		if inSlice(tt.testPath, matches) != tt.shouldMatchGlob {
+		if !g.filesOnly && !g.noFollow && inSlice(tt.testPath, matches) != tt.shouldMatchGlob {
 			if tt.shouldMatchGlob {
 				t.Errorf("#%v. %v(%#q, %#v) = %#v - doesn't contain %v, but should", idx, fn, tt.pattern, g, matches, tt.testPath)
 			} else {
@@ -637,6 +744,44 @@ func compareSlices(a, b []string) bool {
 	return len(diff) == 0
 }
 
+func buildNumResults() {
+	testLen := len(matchTests)
+	numResultsFilesOnly = make([]int, testLen, testLen)
+	numResultsNoFollow = make([]int, testLen, testLen)
+	numResultsAllOpts = make([]int, testLen, testLen)
+
+	fsys := os.DirFS("test")
+	g := newGlob()
+	for idx, tt := range matchTests {
+		if tt.testOnDisk {
+			filesOnly := 0
+			noFollow := 0
+			allOpts := 0
+			GlobWalk(fsys, tt.pattern, func(p string, d fs.DirEntry) error {
+				isDir, _ := g.isDir(fsys, "", p, d)
+				if !isDir {
+					filesOnly++
+				}
+
+				hasNoFollow := (strings.HasPrefix(tt.pattern, "working-symlink") || !strings.Contains(p, "working-symlink/")) && !strings.Contains(p, "/symlink-dir/")
+				if hasNoFollow {
+					noFollow++
+				}
+
+				if hasNoFollow && (!isDir || p == "working-symlink") {
+					allOpts++
+				}
+
+				return nil
+			})
+
+			numResultsFilesOnly[idx] = filesOnly
+			numResultsNoFollow[idx] = noFollow
+			numResultsAllOpts[idx] = allOpts
+		}
+	}
+}
+
 func mkdirp(parts ...string) {
 	dirs := path.Join(parts...)
 	err := os.MkdirAll(dirs, 0755)
@@ -676,9 +821,10 @@ func TestMain(m *testing.M) {
 	mkdirp("test", "axbxcxdxe", "xxx")
 	mkdirp("test", "axbxcxdxexxx")
 	mkdirp("test", "b")
-	mkdirp("test", "e")
+	mkdirp("test", "e", "[x]", "[y]")
 
 	// create test files
+	touch("test", "1")
 	touch("test", "a", "abc")
 	touch("test", "a", "b", "c", "d")
 	touch("test", "a", "c", "b")
@@ -705,6 +851,7 @@ func TestMain(m *testing.M) {
 	touch("test", "e", "{")
 	touch("test", "e", "}")
 	touch("test", "e", "[]")
+	touch("test", "e", "[x]", "[y]", "z")
 
 	touch("test", "}")
 
@@ -729,5 +876,8 @@ func TestMain(m *testing.M) {
 		}
 	}
 
+	// initialize numResultsFilesOnly
+	buildNumResults()
+
 	os.Exit(m.Run())
 }
