@@ -1,5 +1,5 @@
 // FIXME(thaJeztah): remove once we are a module; the go:build directive prevents go from downgrading language version to go1.16:
-//go:build go1.21
+//go:build go1.23
 
 package formatter
 
@@ -12,84 +12,179 @@ import (
 	"time"
 
 	"github.com/docker/cli/internal/test"
-	"github.com/docker/docker/api/types"
-	"github.com/docker/docker/pkg/stringid"
+	"github.com/docker/docker/api/types/container"
+	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 	"gotest.tools/v3/golden"
 )
 
 func TestContainerPsContext(t *testing.T) {
-	containerID := stringid.GenerateRandomID()
+	containerID := test.RandomID()
 	unix := time.Now().Add(-65 * time.Second).Unix()
 
 	var ctx ContainerContext
 	cases := []struct {
-		container types.Container
+		container container.Summary
 		trunc     bool
 		expValue  string
 		call      func() string
 	}{
-		{types.Container{ID: containerID}, true, stringid.TruncateID(containerID), ctx.ID},
-		{types.Container{ID: containerID}, false, containerID, ctx.ID},
-		{types.Container{Names: []string{"/foobar_baz"}}, true, "foobar_baz", ctx.Names},
-		{types.Container{Image: "ubuntu"}, true, "ubuntu", ctx.Image},
-		{types.Container{Image: "verylongimagename"}, true, "verylongimagename", ctx.Image},
-		{types.Container{Image: "verylongimagename"}, false, "verylongimagename", ctx.Image},
 		{
-			types.Container{
+			container: container.Summary{ID: containerID},
+			trunc:     true,
+			expValue:  TruncateID(containerID),
+			call:      ctx.ID,
+		},
+		{
+			container: container.Summary{ID: containerID},
+			expValue:  containerID,
+			call:      ctx.ID,
+		},
+		{
+			container: container.Summary{Names: []string{"/foobar_baz"}},
+			trunc:     true,
+			expValue:  "foobar_baz",
+			call:      ctx.Names,
+		},
+		{
+			container: container.Summary{Image: "ubuntu"},
+			trunc:     true,
+			expValue:  "ubuntu",
+			call:      ctx.Image,
+		},
+		{
+			container: container.Summary{Image: "verylongimagename"},
+			trunc:     true,
+			expValue:  "verylongimagename",
+			call:      ctx.Image,
+		},
+		{
+			container: container.Summary{Image: "verylongimagename"},
+			expValue:  "verylongimagename",
+			call:      ctx.Image,
+		},
+		{
+			container: container.Summary{
 				Image:   "a5a665ff33eced1e0803148700880edab4",
 				ImageID: "a5a665ff33eced1e0803148700880edab4269067ed77e27737a708d0d293fbf5",
 			},
-			true,
-			"a5a665ff33ec",
-			ctx.Image,
+			trunc:    true,
+			expValue: "a5a665ff33ec",
+			call:     ctx.Image,
 		},
 		{
-			types.Container{
+			container: container.Summary{
 				Image:   "a5a665ff33eced1e0803148700880edab4",
 				ImageID: "a5a665ff33eced1e0803148700880edab4269067ed77e27737a708d0d293fbf5",
 			},
-			false,
-			"a5a665ff33eced1e0803148700880edab4",
-			ctx.Image,
+			expValue: "a5a665ff33eced1e0803148700880edab4",
+			call:     ctx.Image,
 		},
-		{types.Container{Image: ""}, true, "<no image>", ctx.Image},
-		{types.Container{Command: "sh -c 'ls -la'"}, true, `"sh -c 'ls -la'"`, ctx.Command},
-		{types.Container{Created: unix}, true, time.Unix(unix, 0).String(), ctx.CreatedAt},
-		{types.Container{Ports: []types.Port{{PrivatePort: 8080, PublicPort: 8080, Type: "tcp"}}}, true, "8080/tcp", ctx.Ports},
-		{types.Container{Status: "RUNNING"}, true, "RUNNING", ctx.Status},
-		{types.Container{SizeRw: 10}, true, "10B", ctx.Size},
-		{types.Container{SizeRw: 10, SizeRootFs: 20}, true, "10B (virtual 20B)", ctx.Size},
-		{types.Container{}, true, "", ctx.Labels},
-		{types.Container{Labels: map[string]string{"cpu": "6", "storage": "ssd"}}, true, "cpu=6,storage=ssd", ctx.Labels},
-		{types.Container{Created: unix}, true, "About a minute ago", ctx.RunningFor},
-		{types.Container{
-			Mounts: []types.MountPoint{
-				{
-					Name:   "this-is-a-long-volume-name-and-will-be-truncated-if-trunc-is-set",
-					Driver: "local",
-					Source: "/a/path",
+		{
+			container: container.Summary{Image: ""},
+			trunc:     true,
+			expValue:  "<no image>",
+			call:      ctx.Image,
+		},
+		{
+			container: container.Summary{Command: "sh -c 'ls -la'"},
+			trunc:     true,
+			expValue:  `"sh -c 'ls -la'"`,
+			call:      ctx.Command,
+		},
+		{
+			container: container.Summary{Created: unix},
+			trunc:     true,
+			expValue:  time.Unix(unix, 0).String(),
+			call:      ctx.CreatedAt,
+		},
+		{
+			container: container.Summary{Ports: []container.Port{{PrivatePort: 8080, PublicPort: 8080, Type: "tcp"}}},
+			trunc:     true,
+			expValue:  "8080/tcp",
+			call:      ctx.Ports,
+		},
+		{
+			container: container.Summary{Status: "Up 123 seconds"},
+			trunc:     true,
+			expValue:  "Up 123 seconds",
+			call:      ctx.Status,
+		},
+		{
+			container: container.Summary{State: container.StateRunning},
+			trunc:     true,
+			expValue:  container.StateRunning,
+			call:      ctx.State,
+		},
+		{
+			container: container.Summary{SizeRw: 10},
+			trunc:     true,
+			expValue:  "10B",
+			call:      ctx.Size,
+		},
+		{
+			container: container.Summary{SizeRw: 10, SizeRootFs: 20},
+			trunc:     true,
+			expValue:  "10B (virtual 20B)",
+			call:      ctx.Size,
+		},
+		{
+			container: container.Summary{},
+			trunc:     true,
+			call:      ctx.Labels,
+		},
+		{
+			container: container.Summary{Labels: map[string]string{"cpu": "6", "storage": "ssd"}},
+			trunc:     true,
+			expValue:  "cpu=6,storage=ssd",
+			call:      ctx.Labels,
+		},
+		{
+			container: container.Summary{Created: unix},
+			trunc:     true,
+			expValue:  "About a minute ago",
+			call:      ctx.RunningFor,
+		},
+		{
+			container: container.Summary{
+				Mounts: []container.MountPoint{
+					{
+						Name:   "this-is-a-long-volume-name-and-will-be-truncated-if-trunc-is-set",
+						Driver: "local",
+						Source: "/a/path",
+					},
 				},
 			},
-		}, true, "this-is-a-long…", ctx.Mounts},
-		{types.Container{
-			Mounts: []types.MountPoint{
-				{
-					Driver: "local",
-					Source: "/a/path",
+			trunc:    true,
+			expValue: "this-is-a-long…",
+			call:     ctx.Mounts,
+		},
+		{
+			container: container.Summary{
+				Mounts: []container.MountPoint{
+					{
+						Driver: "local",
+						Source: "/a/path",
+					},
 				},
 			},
-		}, false, "/a/path", ctx.Mounts},
-		{types.Container{
-			Mounts: []types.MountPoint{
-				{
-					Name:   "733908409c91817de8e92b0096373245f329f19a88e2c849f02460e9b3d1c203",
-					Driver: "local",
-					Source: "/a/path",
+			expValue: "/a/path",
+			call:     ctx.Mounts,
+		},
+		{
+			container: container.Summary{
+				Mounts: []container.MountPoint{
+					{
+						Name:   "733908409c91817de8e92b0096373245f329f19a88e2c849f02460e9b3d1c203",
+						Driver: "local",
+						Source: "/a/path",
+					},
 				},
 			},
-		}, false, "733908409c91817de8e92b0096373245f329f19a88e2c849f02460e9b3d1c203", ctx.Mounts},
+			expValue: "733908409c91817de8e92b0096373245f329f19a88e2c849f02460e9b3d1c203",
+			call:     ctx.Mounts,
+		},
 	}
 
 	for _, c := range cases {
@@ -102,7 +197,7 @@ func TestContainerPsContext(t *testing.T) {
 		}
 	}
 
-	c1 := types.Container{Labels: map[string]string{"com.docker.swarm.swarm-id": "33", "com.docker.swarm.node_name": "ubuntu"}}
+	c1 := container.Summary{Labels: map[string]string{"com.docker.swarm.swarm-id": "33", "com.docker.swarm.node_name": "ubuntu"}}
 	ctx = ContainerContext{c: c1, trunc: true}
 
 	sid := ctx.Label("com.docker.swarm.swarm-id")
@@ -115,7 +210,7 @@ func TestContainerPsContext(t *testing.T) {
 		t.Fatalf("Expected ubuntu, was %s\n", node)
 	}
 
-	c2 := types.Container{}
+	c2 := container.Summary{}
 	ctx = ContainerContext{c: c2, trunc: true}
 
 	label := ctx.Label("anything.really")
@@ -134,52 +229,52 @@ func TestContainerContextWrite(t *testing.T) {
 	}{
 		// Errors
 		{
-			Context{Format: "{{InvalidFunction}}"},
-			`template parsing error: template: :1: function "InvalidFunction" not defined`,
+			context:  Context{Format: "{{InvalidFunction}}"},
+			expected: `template parsing error: template: :1: function "InvalidFunction" not defined`,
 		},
 		{
-			Context{Format: "{{nil}}"},
-			`template parsing error: template: :1:2: executing "" at <nil>: nil is not a command`,
+			context:  Context{Format: "{{nil}}"},
+			expected: `template parsing error: template: :1:2: executing "" at <nil>: nil is not a command`,
 		},
 		// Table Format
 		{
-			Context{Format: NewContainerFormat("table", false, true)},
-			`CONTAINER ID   IMAGE     COMMAND   CREATED        STATUS    PORTS     NAMES        SIZE
+			context: Context{Format: NewContainerFormat("table", false, true)},
+			expected: `CONTAINER ID   IMAGE     COMMAND   CREATED        STATUS    PORTS     NAMES        SIZE
 containerID1   ubuntu    ""        24 hours ago                       foobar_baz   0B
 containerID2   ubuntu    ""        24 hours ago                       foobar_bar   0B
 `,
 		},
 		{
-			Context{Format: NewContainerFormat("table", false, false)},
-			`CONTAINER ID   IMAGE     COMMAND   CREATED        STATUS    PORTS     NAMES
+			context: Context{Format: NewContainerFormat("table", false, false)},
+			expected: `CONTAINER ID   IMAGE     COMMAND   CREATED        STATUS    PORTS     NAMES
 containerID1   ubuntu    ""        24 hours ago                       foobar_baz
 containerID2   ubuntu    ""        24 hours ago                       foobar_bar
 `,
 		},
 		{
-			Context{Format: NewContainerFormat("table {{.Image}}", false, false)},
-			"IMAGE\nubuntu\nubuntu\n",
+			context:  Context{Format: NewContainerFormat("table {{.Image}}", false, false)},
+			expected: "IMAGE\nubuntu\nubuntu\n",
 		},
 		{
-			Context{Format: NewContainerFormat("table {{.Image}}", false, true)},
-			"IMAGE\nubuntu\nubuntu\n",
+			context:  Context{Format: NewContainerFormat("table {{.Image}}", false, true)},
+			expected: "IMAGE\nubuntu\nubuntu\n",
 		},
 		{
-			Context{Format: NewContainerFormat("table {{.Image}}", true, false)},
-			"containerID1\ncontainerID2\n",
+			context:  Context{Format: NewContainerFormat("table {{.Image}}", true, false)},
+			expected: "containerID1\ncontainerID2\n",
 		},
 		{
-			Context{Format: NewContainerFormat("table", true, false)},
-			"containerID1\ncontainerID2\n",
+			context:  Context{Format: NewContainerFormat("table", true, false)},
+			expected: "containerID1\ncontainerID2\n",
 		},
 		{
-			Context{Format: NewContainerFormat("table {{.State}}", false, true)},
-			"STATE\nrunning\nrunning\n",
+			context:  Context{Format: NewContainerFormat("table {{.State}}", false, true)},
+			expected: "STATE\nrunning\nrunning\n",
 		},
 		// Raw Format
 		{
-			Context{Format: NewContainerFormat("raw", false, false)},
-			fmt.Sprintf(`container_id: containerID1
+			context: Context{Format: NewContainerFormat("raw", false, false)},
+			expected: fmt.Sprintf(`container_id: containerID1
 image: ubuntu
 command: ""
 created_at: %s
@@ -202,8 +297,8 @@ ports:
 `, expectedTime, expectedTime),
 		},
 		{
-			Context{Format: NewContainerFormat("raw", false, true)},
-			fmt.Sprintf(`container_id: containerID1
+			context: Context{Format: NewContainerFormat("raw", false, true)},
+			expected: fmt.Sprintf(`container_id: containerID1
 image: ubuntu
 command: ""
 created_at: %s
@@ -228,36 +323,40 @@ size: 0B
 `, expectedTime, expectedTime),
 		},
 		{
-			Context{Format: NewContainerFormat("raw", true, false)},
-			"container_id: containerID1\ncontainer_id: containerID2\n",
+			context:  Context{Format: NewContainerFormat("raw", true, false)},
+			expected: "container_id: containerID1\ncontainer_id: containerID2\n",
 		},
 		// Custom Format
 		{
-			Context{Format: "{{.Image}}"},
-			"ubuntu\nubuntu\n",
+			context:  Context{Format: "{{.Image}}"},
+			expected: "ubuntu\nubuntu\n",
 		},
 		{
-			Context{Format: NewContainerFormat("{{.Image}}", false, true)},
-			"ubuntu\nubuntu\n",
+			context:  Context{Format: NewContainerFormat("{{.Image}}", false, true)},
+			expected: "ubuntu\nubuntu\n",
 		},
 		// Special headers for customized table format
 		{
-			Context{Format: NewContainerFormat(`table {{truncate .ID 5}}\t{{json .Image}} {{.RunningFor}}/{{title .Status}}/{{pad .Ports 2 2}}.{{upper .Names}} {{lower .Status}}`, false, true)},
-			string(golden.Get(t, "container-context-write-special-headers.golden")),
+			context: Context{
+				Format: NewContainerFormat(
+					`table {{truncate .ID 5}}\t{{json .Image}} {{.RunningFor}}/{{title .Status}}/{{pad .Ports 2 2}}.{{upper .Names}} {{lower .Status}}`,
+					false, true,
+				),
+			},
+			expected: string(golden.Get(t, "container-context-write-special-headers.golden")),
 		},
 		{
-			Context{Format: NewContainerFormat(`table {{split .Image ":"}}`, false, false)},
-			"IMAGE\n[ubuntu]\n[ubuntu]\n",
+			context:  Context{Format: NewContainerFormat(`table {{split .Image ":"}}`, false, false)},
+			expected: "IMAGE\n[ubuntu]\n[ubuntu]\n",
 		},
 	}
 
-	containers := []types.Container{
-		{ID: "containerID1", Names: []string{"/foobar_baz"}, Image: "ubuntu", Created: unixTime, State: "running"},
-		{ID: "containerID2", Names: []string{"/foobar_bar"}, Image: "ubuntu", Created: unixTime, State: "running"},
+	containers := []container.Summary{
+		{ID: "containerID1", Names: []string{"/foobar_baz"}, Image: "ubuntu", Created: unixTime, State: container.StateRunning},
+		{ID: "containerID2", Names: []string{"/foobar_bar"}, Image: "ubuntu", Created: unixTime, State: container.StateRunning},
 	}
 
 	for _, tc := range cases {
-		tc := tc
 		t.Run(string(tc.context.Format), func(t *testing.T) {
 			var out bytes.Buffer
 			tc.context.Output = &out
@@ -272,79 +371,90 @@ size: 0B
 }
 
 func TestContainerContextWriteWithNoContainers(t *testing.T) {
-	out := bytes.NewBufferString("")
-	containers := []types.Container{}
-
 	cases := []struct {
 		context  Context
 		expected string
 	}{
 		{
-			Context{
+			context: Context{
 				Format: "{{.Image}}",
-				Output: out,
 			},
-			"",
 		},
 		{
-			Context{
+			context: Context{
 				Format: "table {{.Image}}",
-				Output: out,
 			},
-			"IMAGE\n",
+			expected: "IMAGE\n",
 		},
 		{
-			Context{
+			context: Context{
 				Format: NewContainerFormat("{{.Image}}", false, true),
-				Output: out,
 			},
-			"",
 		},
 		{
-			Context{
+			context: Context{
 				Format: NewContainerFormat("table {{.Image}}", false, true),
-				Output: out,
 			},
-			"IMAGE\n",
+			expected: "IMAGE\n",
 		},
 		{
-			Context{
+			context: Context{
 				Format: "table {{.Image}}\t{{.Size}}",
-				Output: out,
 			},
-			"IMAGE     SIZE\n",
+			expected: "IMAGE     SIZE\n",
 		},
 		{
-			Context{
+			context: Context{
 				Format: NewContainerFormat("table {{.Image}}\t{{.Size}}", false, true),
-				Output: out,
 			},
-			"IMAGE     SIZE\n",
+			expected: "IMAGE     SIZE\n",
 		},
 	}
 
 	for _, tc := range cases {
-		tc := tc
 		t.Run(string(tc.context.Format), func(t *testing.T) {
-			err := ContainerWrite(tc.context, containers)
+			out := new(bytes.Buffer)
+			tc.context.Output = out
+			err := ContainerWrite(tc.context, nil)
 			assert.NilError(t, err)
 			assert.Equal(t, out.String(), tc.expected)
-			// Clean buffer
-			out.Reset()
 		})
 	}
 }
 
 func TestContainerContextWriteJSON(t *testing.T) {
 	unix := time.Now().Add(-65 * time.Second).Unix()
-	containers := []types.Container{
-		{ID: "containerID1", Names: []string{"/foobar_baz"}, Image: "ubuntu", Created: unix, State: "running"},
-		{ID: "containerID2", Names: []string{"/foobar_bar"}, Image: "ubuntu", Created: unix, State: "running"},
+	containers := []container.Summary{
+		{
+			ID:      "containerID1",
+			Names:   []string{"/foobar_baz"},
+			Image:   "ubuntu",
+			Created: unix,
+			State:   container.StateRunning,
+		},
+		{
+			ID:      "containerID2",
+			Names:   []string{"/foobar_bar"},
+			Image:   "ubuntu",
+			Created: unix,
+			State:   container.StateRunning,
+
+			ImageManifestDescriptor: &ocispec.Descriptor{Platform: &ocispec.Platform{Architecture: "amd64", OS: "linux"}},
+		},
+		{
+			ID:      "containerID3",
+			Names:   []string{"/foobar_bar"},
+			Image:   "ubuntu",
+			Created: unix,
+			State:   container.StateRunning,
+
+			ImageManifestDescriptor: &ocispec.Descriptor{Platform: &ocispec.Platform{}},
+		},
 	}
 	expectedCreated := time.Unix(unix, 0).String()
 	expectedJSONs := []map[string]any{
 		{
-			"Command":      "\"\"",
+			"Command":      `""`,
 			"CreatedAt":    expectedCreated,
 			"ID":           "containerID1",
 			"Image":        "ubuntu",
@@ -353,6 +463,7 @@ func TestContainerContextWriteJSON(t *testing.T) {
 			"Mounts":       "",
 			"Names":        "foobar_baz",
 			"Networks":     "",
+			"Platform":     nil,
 			"Ports":        "",
 			"RunningFor":   "About a minute ago",
 			"Size":         "0B",
@@ -360,7 +471,7 @@ func TestContainerContextWriteJSON(t *testing.T) {
 			"Status":       "",
 		},
 		{
-			"Command":      "\"\"",
+			"Command":      `""`,
 			"CreatedAt":    expectedCreated,
 			"ID":           "containerID2",
 			"Image":        "ubuntu",
@@ -369,6 +480,24 @@ func TestContainerContextWriteJSON(t *testing.T) {
 			"Mounts":       "",
 			"Names":        "foobar_bar",
 			"Networks":     "",
+			"Platform":     map[string]any{"architecture": "amd64", "os": "linux"},
+			"Ports":        "",
+			"RunningFor":   "About a minute ago",
+			"Size":         "0B",
+			"State":        "running",
+			"Status":       "",
+		},
+		{
+			"Command":      `""`,
+			"CreatedAt":    expectedCreated,
+			"ID":           "containerID3",
+			"Image":        "ubuntu",
+			"Labels":       "",
+			"LocalVolumes": "0",
+			"Mounts":       "",
+			"Names":        "foobar_bar",
+			"Networks":     "",
+			"Platform":     map[string]any{"architecture": "", "os": ""},
 			"Ports":        "",
 			"RunningFor":   "About a minute ago",
 			"Size":         "0B",
@@ -391,7 +520,7 @@ func TestContainerContextWriteJSON(t *testing.T) {
 }
 
 func TestContainerContextWriteJSONField(t *testing.T) {
-	containers := []types.Container{
+	containers := []container.Summary{
 		{ID: "containerID1", Names: []string{"/foobar_baz"}, Image: "ubuntu"},
 		{ID: "containerID2", Names: []string{"/foobar_bar"}, Image: "ubuntu"},
 	}
@@ -410,89 +539,119 @@ func TestContainerContextWriteJSONField(t *testing.T) {
 }
 
 func TestContainerBackCompat(t *testing.T) {
-	containers := []types.Container{{ID: "brewhaha"}}
-	cases := []string{
-		"ID",
-		"Names",
-		"Image",
-		"Command",
-		"CreatedAt",
-		"RunningFor",
-		"Ports",
-		"Status",
-		"Size",
-		"Labels",
-		"Mounts",
+	createdAtTime := time.Now().AddDate(-1, 0, 0) // 1 year ago
+
+	ctrContext := container.Summary{
+		ID:                      "aabbccddeeff",
+		Names:                   []string{"/foobar_baz"},
+		Image:                   "docker.io/library/ubuntu",                                                // should this have canonical format or not?
+		ImageID:                 "sha256:a5a665ff33eced1e0803148700880edab4269067ed77e27737a708d0d293fbf5", // should this have algo-prefix or not?
+		ImageManifestDescriptor: nil,
+		Command:                 "/bin/sh",
+		Created:                 createdAtTime.UTC().Unix(),
+		Ports:                   []container.Port{{PrivatePort: 8080, PublicPort: 8080, Type: "tcp"}},
+		SizeRw:                  123,
+		SizeRootFs:              12345,
+		Labels:                  map[string]string{"label1": "value1", "label2": "value2"},
+		State:                   "running",
+		Status:                  "running",
+		HostConfig: struct {
+			NetworkMode string            `json:",omitempty"`
+			Annotations map[string]string `json:",omitempty"`
+		}{
+			NetworkMode: "bridge",
+			Annotations: map[string]string{
+				"com.example.annotation": "hello",
+			},
+		},
+		NetworkSettings: nil,
+		Mounts:          nil,
 	}
-	buf := bytes.NewBuffer(nil)
-	for _, c := range cases {
-		ctx := Context{Format: Format(fmt.Sprintf("{{ .%s }}", c)), Output: buf}
-		if err := ContainerWrite(ctx, containers); err != nil {
-			t.Logf("could not render template for field '%s': %v", c, err)
-			t.Fail()
-		}
-		buf.Reset()
+
+	tests := []struct {
+		field    string
+		expected string
+	}{
+		{field: "ID", expected: "aabbccddeeff"},
+		{field: "Names", expected: "foobar_baz"},
+		{field: "Image", expected: "docker.io/library/ubuntu"},
+		{field: "Command", expected: `"/bin/sh"`},
+		{field: "CreatedAt", expected: time.Unix(createdAtTime.Unix(), 0).String()},
+		{field: "RunningFor", expected: "12 months ago"},
+		{field: "Ports", expected: "8080/tcp"},
+		{field: "Status", expected: "running"},
+		{field: "Size", expected: "123B (virtual 12.3kB)"},
+		{field: "Labels", expected: "label1=value1,label2=value2"},
+		{field: "Mounts", expected: ""},
+	}
+
+	for _, tc := range tests {
+		t.Run(tc.field, func(t *testing.T) {
+			buf := new(bytes.Buffer)
+			ctx := Context{Format: Format(fmt.Sprintf("{{ .%s }}", tc.field)), Output: buf}
+			assert.NilError(t, ContainerWrite(ctx, []container.Summary{ctrContext}))
+			assert.Check(t, is.Equal(strings.TrimSpace(buf.String()), tc.expected))
+		})
 	}
 }
 
 type ports struct {
-	ports    []types.Port
+	ports    []container.Port
 	expected string
 }
 
-//nolint:lll
 func TestDisplayablePorts(t *testing.T) {
 	cases := []ports{
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					PrivatePort: 9988,
 					Type:        "tcp",
 				},
 			},
-			"9988/tcp",
+			expected: "9988/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					PrivatePort: 9988,
 					Type:        "udp",
 				},
 			},
-			"9988/udp",
+			expected: "9988/udp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					IP:          "0.0.0.0",
 					PrivatePort: 9988,
 					Type:        "tcp",
 				},
 			},
-			"0.0.0.0:0->9988/tcp",
+			expected: "0.0.0.0:0->9988/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					IP:          "::",
 					PrivatePort: 9988,
 					Type:        "tcp",
 				},
 			},
-			"[::]:0->9988/tcp",
+			expected: "[::]:0->9988/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					PrivatePort: 9988,
 					PublicPort:  8899,
 					Type:        "tcp",
 				},
 			},
-			"9988/tcp",
+			expected: "9988/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					IP:          "4.3.2.1",
 					PrivatePort: 9988,
@@ -500,10 +659,21 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "tcp",
 				},
 			},
-			"4.3.2.1:8899->9988/tcp",
+			expected: "4.3.2.1:8899->9988/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
+				{
+					IP:          "::1",
+					PrivatePort: 9988,
+					PublicPort:  8899,
+					Type:        "tcp",
+				},
+			},
+			expected: "[::1]:8899->9988/tcp",
+		},
+		{
+			ports: []container.Port{
 				{
 					IP:          "4.3.2.1",
 					PrivatePort: 9988,
@@ -511,10 +681,21 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "tcp",
 				},
 			},
-			"4.3.2.1:9988->9988/tcp",
+			expected: "4.3.2.1:9988->9988/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
+				{
+					IP:          "::1",
+					PrivatePort: 9988,
+					PublicPort:  9988,
+					Type:        "tcp",
+				},
+			},
+			expected: "[::1]:9988->9988/tcp",
+		},
+		{
+			ports: []container.Port{
 				{
 					PrivatePort: 9988,
 					Type:        "udp",
@@ -523,10 +704,10 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "udp",
 				},
 			},
-			"9988/udp, 9988/udp",
+			expected: "9988/udp, 9988/udp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					IP:          "1.2.3.4",
 					PublicPort:  9998,
@@ -539,10 +720,26 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "udp",
 				},
 			},
-			"1.2.3.4:9998-9999->9998-9999/udp",
+			expected: "1.2.3.4:9998-9999->9998-9999/udp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
+				{
+					IP:          "::1",
+					PublicPort:  9998,
+					PrivatePort: 9998,
+					Type:        "udp",
+				}, {
+					IP:          "::1",
+					PublicPort:  9999,
+					PrivatePort: 9999,
+					Type:        "udp",
+				},
+			},
+			expected: "[::1]:9998-9999->9998-9999/udp",
+		},
+		{
+			ports: []container.Port{
 				{
 					IP:          "1.2.3.4",
 					PublicPort:  8887,
@@ -555,10 +752,26 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "udp",
 				},
 			},
-			"1.2.3.4:8887->9998/udp, 1.2.3.4:8888->9999/udp",
+			expected: "1.2.3.4:8887->9998/udp, 1.2.3.4:8888->9999/udp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
+				{
+					IP:          "::1",
+					PublicPort:  8887,
+					PrivatePort: 9998,
+					Type:        "udp",
+				}, {
+					IP:          "::1",
+					PublicPort:  8888,
+					PrivatePort: 9999,
+					Type:        "udp",
+				},
+			},
+			expected: "[::1]:8887->9998/udp, [::1]:8888->9999/udp",
+		},
+		{
+			ports: []container.Port{
 				{
 					PrivatePort: 9998,
 					Type:        "udp",
@@ -567,10 +780,10 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "udp",
 				},
 			},
-			"9998-9999/udp",
+			expected: "9998-9999/udp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					IP:          "1.2.3.4",
 					PrivatePort: 6677,
@@ -582,10 +795,10 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "udp",
 				},
 			},
-			"9988/udp, 1.2.3.4:7766->6677/tcp",
+			expected: "9988/udp, 1.2.3.4:7766->6677/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					IP:          "1.2.3.4",
 					PrivatePort: 9988,
@@ -601,12 +814,17 @@ func TestDisplayablePorts(t *testing.T) {
 					PrivatePort: 2233,
 					PublicPort:  3322,
 					Type:        "tcp",
+				}, {
+					IP:          "::1",
+					PrivatePort: 2233,
+					PublicPort:  3322,
+					Type:        "tcp",
 				},
 			},
-			"4.3.2.1:3322->2233/tcp, 1.2.3.4:8899->9988/tcp, 1.2.3.4:8899->9988/udp",
+			expected: "4.3.2.1:3322->2233/tcp, [::1]:3322->2233/tcp, 1.2.3.4:8899->9988/tcp, 1.2.3.4:8899->9988/udp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					PrivatePort: 9988,
 					PublicPort:  8899,
@@ -623,10 +841,10 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "tcp",
 				},
 			},
-			"9988/udp, 4.3.2.1:3322->2233/tcp, 1.2.3.4:7766->6677/tcp",
+			expected: "9988/udp, 4.3.2.1:3322->2233/tcp, 1.2.3.4:7766->6677/tcp",
 		},
 		{
-			[]types.Port{
+			ports: []container.Port{
 				{
 					PrivatePort: 80,
 					Type:        "tcp",
@@ -684,7 +902,7 @@ func TestDisplayablePorts(t *testing.T) {
 					Type:        "sctp",
 				},
 			},
-			"80/tcp, 80/udp, 1024/tcp, 1024/udp, 12345/sctp, 1.1.1.1:1024->80/tcp, 1.1.1.1:1024->80/udp, 2.1.1.1:1024->80/tcp, 2.1.1.1:1024->80/udp, 1.1.1.1:80->1024/tcp, 1.1.1.1:80->1024/udp, 2.1.1.1:80->1024/tcp, 2.1.1.1:80->1024/udp",
+			expected: "80/tcp, 80/udp, 1024/tcp, 1024/udp, 12345/sctp, 1.1.1.1:1024->80/tcp, 1.1.1.1:1024->80/udp, 2.1.1.1:1024->80/tcp, 2.1.1.1:1024->80/udp, 1.1.1.1:80->1024/tcp, 1.1.1.1:80->1024/udp, 2.1.1.1:80->1024/tcp, 2.1.1.1:80->1024/udp", //nolint:revive // ignore line-length-limit (revive)
 		},
 	}
 
